// forint_types.hh
/****************************************************************************

    C-formula interpreter library forint
    Copyright (C) 2000 Alexey Filin

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
    02111-1307, USA.
              
 ****************************************************************************/

#ifndef _FORINT_TYPES_HH_
#define _FORINT_TYPES_HH_

#include <errno.h>

namespace FInt {

  //#define FINT_DEBUG_PRINT

typedef unsigned int uint;
typedef long long    Llong;

// stack/vec dynamic memory reallocation step
const uint FI_REALLOC_SIZE = 30;

// the biggest number of digit in a decimal number representation
const uint FINT_NUMBER_BUFFER = 100;

// FI - ForInt (or FIlin Alexey ;-) )
enum b_type_t { FI_CHAR, FI_SHORT, FI_INT, FI_LONG, FI_LONG_LONG,
		FI_FLOAT, FI_DOUBLE, FI_LONG_DOUBLE, FI_USR_CLASS };
enum calc_mode_t { RVM_MODE, SVM_MODE, DLF_MODE };
enum parse_mode_t { NEW_FORM_MODE, ADD_FORM_MODE };
enum angle_unit_t { FI_RADIAN, FI_DEGREE, FI_GRAD };

// formula_interpreter's internal language commands and some
// auxiliary symbols.
enum action_t { FI_COOR = 0, FI_X, FI_Y, FI_Z, FI_PHI, FI_PSI, FI_R, FI_RAD,
		             FI_T, FI_U, FI_V, FI_RL, FI_SELF,
		FI_VRBL,
		FI_POIN,
		FI_NUMB, FI_EN_E, FI_EN_LOG2E, FI_EN_LOG10E, FI_EN_LN2,
		         FI_EN_LN10, FI_EN_PI, FI_EN_PI_2, FI_EN_PI_4,
		         FI_EN_1_PI, FI_EN_2_PI, FI_EN_2_SQRTPI,
		         FI_EN_SQRT2, FI_EN_SQRT1_2,
		FI_FUNC, FI_FUNC1, FI_FUNC2D, FI_FUNC2P, FI_FUNC2,
		         FI_SIN, FI_COS, FI_TAN, FI_ASIN, FI_ACOS, FI_ATAN,
		         FI_SINH, FI_COSH, FI_TANH, FI_ASINH, FI_ACOSH, FI_ATANH,
		         FI_EXP, FI_LOG, FI_LOG10, FI_SQRT,
		         FI_EXPM1, FI_LOG1P, FI_CBRT,
		         FI_J0, FI_J1, FI_Y0, FI_Y1, FI_ERF, FI_ERFC, FI_LGAMMA,
		         FI_FABS, FI_ABS, FI_LABS, FI_CEIL, FI_FLOOR, FI_RINT,
		         FI_NOT, FI_COMP, FI_UN_MINUS, FI_UN_PLUS,
		FI_OPER, FI_ADD, FI_SUBT, FI_MULT, FI_DIV, FI_MOD,
                         FI_BAND, FI_BOR, FI_BXOR, FI_LOFF, FI_ROFF,
		         FI_ATAN2, FI_FMOD, FI_POW, FI_HYPOT,
		         FI_LDEXP, FI_DREM, FI_COPYSIGN, FI_JN, FI_YN,
		FI_COND, FI_GT, FI_GE, FI_LT, FI_LE, FI_EQ, FI_NE,
		FI_LOGC, FI_AND, FI_OR,
		FI_GRAM, FI_IF, FI_ELSE, FI_WHILE, FI_CONTINUE, FI_JUMP,
                         FI_SEMICOLON, FI_COMMA,
		         FI_L_PARENTHESIS, FI_R_PARENTHESIS,
                FI_ASS,  FI_ASS_PL, FI_ASS_MIN, FI_ASS_MUL, FI_ASS_DIV,
                         FI_ASS_MOD, FI_ASS_AND, FI_ASS_OR, FI_ASS_XOR,
                         FI_ASS_LOFF, FI_ASS_ROFF };

typedef double (*USR_FUNC_0_PTR) (void);
typedef double (*USR_FUNC_1_PTR) (double);
typedef double (*USR_FUNC_2D_PTR)(double,double);
typedef double (*USR_FUNC_2P_PTR) (void*,void*);
typedef double (*USR_FUNC_2_PTR) (double,void*);

}

#endif //_FORINT_TYPES_HH_
